Struct segvec::Slice[][src]

pub struct Slice<'a, T: 'a> { /* fields omitted */ }
Provides an immutable view of elements from a range in SegVec.

Implementations

Returns the number of elements in the Slice.

Returns an iterator over immutable references to the elements of the Slice.